function [T,dT] = holland_T(r,beta)
% function [T,dT] = holland_T(r,beta)
% Holland parametric temperature profile
% beta = [ delT rm b Tc ]
% dT(:,1:4) is the derivatives of T with respect to beta 
% dT(:,5) is the derivative of T with respect to r

if nargin == 0
   help holland_T
   T = nan;
   return
end

delT = beta(1);
rm = beta(2);
b  = beta(3);
Tc = beta(4);

nr = length(r);
T = zeros(nr,1);
if nargout >= 2
   dT = zeros(nr,5);
end

ix = find(r > 0.0);
if ~isempty(ix)
   rmrb = (rm./r(ix)).^b;
   ermrb = exp(-rmrb);
   T(ix) = Tc - delT*ermrb;
   if nargout >= 2
      dT(ix,1) = -ermrb;
      dT(ix,2) = (delT*b/rm) * ermrb .* rmrb;
      dT(ix,3) =  delT * ermrb .* rmrb .* log(rm./r(ix));
      dT(ix,4) = 1;
      dT(ix,5) = -(delT*b) .* ermrb .* rmrb ./ r(ix);
   end
end

ix = find(r == 0.0);
if ~isempty(ix)
   T(ix) = Tc;
   if nargout >= 2
      dT(ix,1) = 0.0;
      dT(ix,2) = 0.0;
      dT(ix,3) = 0.0;
      dT(ix,4) = 1.0;
      dT(ix,5) = 0.0;
   end
end

ix = find(r == -999.0);
if ~isempty(ix) 
   % Special point, put in penalty term on length scales etc.
   rmhat = 25e3; rmscal = 1;
   bhat = 1.5; bscal = 1;
   T(ix) = rmscal*(rm/rmhat - 1)^4 + bscal*(b/bhat - 1)^4;
   if nargout >= 2
      dT(ix,1) = 0;
      dT(ix,2) = 4*rmscal/rmhat*(rm/rmhat - 1)^3;
      dT(ix,3) = 4*bscal/bhat*(b/bhat - 1)^3;
      dT(ix,4) = 0;
      dT(ix,5) = nan;
   end
end

