function [p,dp] = holland_p(r,beta)
% function [p,dp] = holland_p(r,beta)
% Holland parametric pressure profile
% beta = [ vm rm b pc Tbar ]
% dp(:,1:5) is the derivatives of p with respect to beta 
% dp(:,6) is the derivative of p with respect to r

if nargin < 1
   help holland_p
   p = nan; dp = nan;
   return
end

vm = beta(1);
rm = beta(2);
b  = beta(3);
pc = beta(4);
RT = 287*beta(5);

nr = length(r);
p = zeros(nr,1);
if nargout >= 2
   dp = zeros(nr,6);
end

ix = find(r > 0.0);
if ~isempty(ix)
   rmrb = (rm./r(ix)).^b;
   e1rmrb = exp(1.0 - rmrb);
   p(ix) = pc*( 1.0 + (vm^2 / (b*RT - vm^2)) * e1rmrb );
   if nargout >= 2
      dp(ix,1) = 2*b*pc*RT*vm/(b*RT - vm^2)^2 * e1rmrb;
      dp(ix,2) = -b*pc*vm^2/(rm*(b*RT - vm^2)) * e1rmrb .* rmrb;
      dp(ix,3) = -pc*vm^2/(b*RT - vm^2)^2 ...
         * e1rmrb .* (RT + (b*RT - vm^2)*rmrb.*log(rm./r(ix)));
      dp(ix,4) = 1 + (vm^2 / (b*RT - vm^2)) * e1rmrb;
      dp(ix,5) = nan;
      dp(ix,6) = b*pc*vm^2/(b*RT - vm^2) .* e1rmrb .* rmrb ./ r(ix);
   end
end

ix = find(r == 0.0);
if ~isempty(ix)
   p(ix) = pc;
   if nargout >= 2
      dp(ix,1) = 0.0;
      dp(ix,2) = 0.0;
      dp(ix,3) = 0.0;
      dp(ix,4) = 1.0;
      dp(ix,5) = nan;
      dp(ix,6) = 0.0;
   end
end

ix = find(r == -999.0);
if ~isempty(ix) 
   % Special point, put in penalty term on length scales etc.
   rmhat = 25e3; rmscal = 1;
   bhat = 1.5; bscal = 1;
   p(ix) = rmscal*(rm/rmhat - 1)^4 + bscal*(b/bhat - 1)^4;
   if nargout >= 2
      dp(ix,1) = 0;
      dp(ix,2) = 4*rmscal/rmhat*(rm/rmhat - 1)^3;
      dp(ix,3) = 4*bscal/bhat*(b/bhat - 1)^3;
      dp(ix,4) = 0;
      dp(ix,5) = nan;
      dp(ix,6) = nan;
   end
end

