function [v,dv,wzone] = mrankine_v(r,a)
% function [v,dv,wzone] = mrankine_v(r,a)
% Returns the double-exponential parametric wind profile of Willoughby (2002)
% at radius points r for control vector a=(A,L1,B,L2,rm,n1,pc,T, Lb) and its derivatives
% with respect to the control vector. Also returns wzone which contains the
% radii at which the blending starts and finishes.
% Feb 2006 modified from willoughby2_v.m

%% Note not fully tested

persistent olda vm rm alp n1 pc RT Lb wm dwmdvm dwmdrm dwmdalp dwmdn1
persistent rsm dwsdrm_rsm dwsdrsi_rsm

if isempty(olda) | any(size(a) ~= size(olda)) | any(a ~= olda)  
   % Initialise on first time or changed a
   olda = a;
   
   vm  = a(1);   % maximum wind
   rm  = a(2);   % radius of maximum winds
   alp = a(3);   % outer vortex decay exponent   ( v2(r) = vm*(r/rm)^(-alp) )
   n1  = a(4);   % Exponent for v1 (inner zone)  ( v1(r) = vm*(r/rm)^n1 )
   pc  = a(5);   % central pressure (not used)
   RT  = 287*a(6);  % for density terms (not used)
   Lb  = a(7);   % Half-width of blending zone
   
   % Find wm, the weight that must apply at rm, and its derivatives.
   wm = alp/(n1 + alp);
   dwmdvm  = 0;
   dwmdrm  = 0;
   dwmdalp = n1 / (n1 + alp)^2;
   dwmdn1  = -alp / (n1 + alp)^2;
   
   % Find the scaled radius rsm = rs(rm) satisfying ws(rsm) = wm, by bisection
   rsm = mrankine_rsm(a);

   if ( rm - Lb*rsm - Lb  < 0 )
      disp(sprintf('WARNING mrankine_v >> rblend1 = %f < 0, rsm adjusted', rm - Lb*rsm - Lb));
      rsm = rm/Lb - 1;
   end
   
   dwsdrs_rsm = (315/256)*(rsm^2 - 1)^4;
   dwsdrsi_rsm = 1/dwsdrs_rsm;
end

rs = ( r - rm )/Lb + rsm;   % Scaled r for blending

nr  = length(r); na = 7;

w  = zeros(size(r)); 

ix = find(abs(rs) < 1);
w(ix) = (1/256) * (1 + rs(ix)).^5 .* (128 - rs(ix).*(325 - rs(ix).*(345 - rs(ix).*(175 - rs(ix)*35))));
w( find(rs >= 1) ) = 1;

rrm = r/rm;

ix = find(r ~= -999.0);  % Normal points

rrmn1  = rrm.^n1;
rrmalp = rrm.^(-alp);
v1 = vm*rrmn1;
v2 = vm*rrmalp;
v = (1 - w).*v1 + w.*v2;

if nargout >= 2
   ix = find(abs(rs) < 1);
   dwsdrs_rs = (315/256)*(rs(ix).^2 - 1).^4;
   term = dwsdrs_rs * dwsdrsi_rsm;
   dw  = zeros(nr,na+1);
   dw(ix, 1) = 0;
   dw(ix, 2) = 0;
   dw(ix, 3) = term * dwmdalp;
   dw(ix, 4) = term * dwmdn1;
   dw(ix, 5) = 0;
   dw(ix, 6) = 0;
   dw(ix, 7) = nan;
   dw(ix, 8) = dwsdrs_rs/Lb;
   dw( find(rs >= 1) ,:) = 0;

   dv1 = zeros(nr,na+1);
   dv1(:, 1) = rrmn1;
   dv1(:, 2) = -(n1/rm)*v1;
   dv1(:, 3) = 0;
   ix = find(r >  0); dv1(ix, 4) = log(rrm(ix)).*v1(ix);
   ix = find(r == 0); dv1(ix, 4) = 0;
   dv1(:, 5) = 0;
   dv1(:, 6) = 0;
   dv1(:, 7) = 0;
   dv1(:, 8) = (n1./r(ix))*v1;
   
   dv2 = zeros(nr,na+1);
   dv2(:, 1) = rrmalp;
   dv2(:, 2) = (alp/rm)*v2;
   ix = find(r >  0); dv2(ix, 3) = -log(rrm(ix)).*v2(ix);
   ix = find(r == 0); dv2(ix, 3) = 0;
   dv2(:, 4) = 0;
   dv2(:, 5) = 0;
   dv2(:, 6) = 0;
   dv2(:, 7) = 0;
   dv2(:, 8) = -(alp/r(ix))*v2;

   oa = ones(1,na+1);
   dv = (1 - w*oa).*dv1 + w*oa.*dv2 + dw.*((v2 - v1)*oa);
end

ix = find(r == -999.0);
if ~isempty(ix) 
   % Special point, put in penalty term on length scales etc.
   global w2v_rmhat w2v_rmscal w2v_L1hat w2v_L1scal w2v_L2hat w2v_L2scal 
   v(ix) = w2v_rmscal*(rm/w2v_rmhat - 1)^2 ...
      + w2v_L1scal*(L1/w2v_L1hat - 1)^4 ...
      + w2v_L2scal*(L2/w2v_L2hat - 1)^4;
   if nargout >= 2
      dv(ix,1) = 0;
      dv(ix,2) = 4*w2v_L1scal/w2v_L1hat*(L1/w2v_L1hat - 1)^3;
      dv(ix,3) = 0;
      dv(ix,4) = 4*w2v_L2scal/w2v_L2hat*(L2/w2v_L2hat - 1)^3;
      dv(ix,5) =   w2v_rmscal/w2v_rmhat*(rm/w2v_rmhat - 1);
      dv(ix,6) = 0;
      dv(ix,7) = 0;
      dv(ix,8) = 0;
   end
end


