function [a, chisq, nf] = nlinfit_amoeba(x, y, sig, a0, dela, funk, ftol, nfmax)

% Multidimensional least squares fit to the function y(i) = funk(x(i),a) 
% where x[1..ndim] is a vector in ndim dimensions, by the downhill 
% simplex method of Nelder and Mead (ref Numerical Recipes in F).
% Input:
%  x[1..n] The observation points
%  y[1..n] The observations
%  sig[1..n] The observation error standard deviations
%  a0[1..ndim]: Initial guess for the control variable.
%  dela[1..ndim]: Estimated uncertainty scale in a0. Any zero elements are taken
%                 to indicate initial guess should not be perturbed there.
% Output:
%	a: Fitted control variable.
%  chisq: Goodness of fit.
%	nf : no of evaluations of f made.

if (nargin <= 3)
   nfmax = 2000;
end

ndim = length(a0); 
% mpts = ndim+1;
xa = find(dela ~= 0);
mpts = length(xa) + 1;
n = length(x);
p = zeros(ndim,mpts);
if 1
   p(:,mpts) = a0 - dela;
   for ii = 1 : mpts-1
      p(:,ii) = a0 + 0.5*dela.*randn(ndim,1); 
      xai = xa(ii);
      p(xai,ii) = p(xai,ii) + dela(xai);
   end
else
   for ii = 1 : mpts
      p(:,ii) = a0 + dela.*randn(ndim,1);
   end
end;
sigi2 = 1.0 ./ sig.^2;
fval = zeros(1,mpts);
for ii = 1 : mpts
   yhat = feval(funk, x, p(:,ii));
   fval(ii) = sum( sigi2 .* (yhat - y).^2 ); % /n;
end
[ p ; fval ]
nmax = 20;
alpha = 1.0;
beta = 0.5;
gamma = 2.0;
nf = 0;

while nf < nfmax
   ilo = 1;
   if fval(1) > fval(2)
      ihi = 1; inhi = 2;
   else
      ihi = 2; inhi = 1;
   end
   for ii = 1 : mpts
      if fval(ii) < fval(ilo)
         ilo = ii;
      end
      if fval(ii) > fval(ihi)
         inhi = ihi;
         ihi = ii;
      elseif fval(ii) > fval(inhi) & ii ~= ihi
         inhi = ii;
      end
   end
   rtol = 2.*abs(fval(ihi)-fval(ilo))/(abs(fval(ihi))+abs(fval(ilo)) + eps);
   if rtol < ftol
      temp = fval(1);   fval(1)   = fval(ilo);   fval(ilo)   = temp;
      vtmp = p(:,1); p(:,1) = p(:,ilo); p(:,ilo) = vtmp;
      [chisq,ii] = min(fval); a = p(:,ii); 
      return
   end
   pbar = ( sum(p,2) - p(:,ihi) ) / ndim;
   pr   = (1 + alpha)*pbar - alpha*p(:,ihi);
   yhat = feval(funk, x, pr);
   ypr = sum( sigi2 .* (yhat - y).^2 ); % /n;
   nf = nf + 1;
   if (isnan(ypr))
      disp('funk undefined');
      return
   end
   if ypr <= fval(ilo)
      prr = gamma*pr + (1 - gamma)*pbar;
      yhat = feval(funk, x, prr);
      yprr = sum( sigi2 .* (yhat - y).^2 ); % /n;
      nf = nf + 1;
      if (isnan(yprr))
         disp('funk undefined');
         return
      end
      if yprr < fval(ilo)
         p(:,ihi) = prr;
         fval(ihi) = yprr;
      else
         p(:,ihi) = pr;
         fval(ihi) = ypr;
      end
   elseif ypr >= fval(inhi)
      if (ypr < fval(ihi))
         p(:,ihi) = pr;
         fval(ihi) = ypr;
      end
      prr = beta*p(:,ihi) + (1 - beta)*pbar;
      yhat = feval(funk, x, prr);
      yprr = sum( sigi2 .* (yhat - y).^2 ); % /n;
      nf = nf + 1;
      if (isnan(yprr))
         disp('funk undefined');
         return
      end
      if (yprr < fval(ihi))
         p(:,ihi) = prr;
         fval(ihi) = yprr;
      else
         for ii = 1 : mpts
            if (ii ~= ilo)
               pr = 0.5*(p(:,ii) + p(:,ilo));
               p(:,ii) = pr;
               yhat = feval(funk, x, pr);
               fval(ii) = sum( sigi2 .* (yhat - y).^2 ); % /n;
               nf = nf + 1;
               if (isnan(fval(ii)))
                  disp('funk undefined');
                  return
               end
            end
         end
      end
   else
      p(:,ihi) = pr;
      fval(ihi) = ypr;
   end
end
disp(sprintf('amoeba exceeding maximum iterations %d.',nf))
[chisq,ii] = min(fval);
a = p(:,ii);
