function [v,dv,wzone] = willoughby1_v(r,a)
% function [v,dv,wzone] = willoughby1_v(r,a)
% Returns the double-exponential parametric wind profile of Willoughby (2002)
% at radius points r for control vector a=(A,L1,B,L2,rm) and its derivatives
% with respect to the control vector. Also returns wzone which contains the
% radii at which the blending starts and finishes.

persistent olda vm L Li rm Lb wm dwmdvm dwmdL dwmdrm 
persistent rsm dwsdrm_rsm dwsdrsi_rsm

a = a(1:3);

if isempty(olda) | any(a ~= olda)  % Initialise on first time or changed a
   olda = a;
   
   vm = a(1);    % amplitude of first exponential
   L  = a(2);   % length scale of first exponential
   rm = a(3);   % radius of maximum winds
   Lb = 5e3;    % Half-width of blending zone
   Li = 1/L;
   
   % Find wm, the weight that must apply at rm, and its derivatives.
   wm     = 1 / ( 1 + rm*Li );
   dwmdvm = 0;
   dwmdL  =  wm^2 * rm * Li^2;
   dwmdrm = -wm^2 * Li; 
   
   % Find the scaled radius rsm = rs(rm) satisfying ws(rsm) = wm, by bisection
   rsm = willoughby1_rsm(a);
   
%% dwsdrs_rsm = 0.9375*(rsm^2 - 1)^2;
   dwsdrs_rsm = (315/256) * (rsm^2 - 1)^4;
   dwsdrsi_rsm = 1/dwsdrs_rsm;

   if ( rm - Lb*rsm - Lb  < 0 )
      disp(sprintf('WARNING willoughby1_v >> rblend1 = %f < 0', rblend1));
      v = repmat(nan, size(r));
      return;
   end

end

rs = ( r - rm )/Lb + rsm;   % Scaled r for blending

nr = length(r); na = length(a);

w  = zeros(nr,1); dw  = zeros(nr,na);
oa = ones(1,na);

ix = find(abs(rs) < 1);
%% Old version has 2 continuous derivatives, new one has 4. (See also above)
%% w(ix) = 0.5 + 0.0625*rs(ix).*(3*rs(ix).^4 - 10*rs(ix).^2 + 15);
%% dwsdrs_rs = 0.9375*(rs(ix).^2 - 1).^2;
w(ix) = (1/256) * (1 + rs(ix)).^5 .* (128 - rs(ix).*(325 - rs(ix).*(345 - rs(ix).*(175 - rs(ix)*35))));
dwsdrs_rs = (315/256) * (rs(ix).^2 - 1).^4;

term = dwsdrs_rs * dwsdrsi_rsm;
dw(ix,1) = term * dwmdvm; 
dw(ix,2) = term * dwmdL; 
dw(ix,3) = term * dwmdrm - dwsdrs_rs/Lb;

ix = find(rs >= 1);
w(ix) = 1;
dw(ix,:) = 0;

v1 = zeros(nr,1); dv1 = zeros(nr,na);
v2 = zeros(nr,1); dv2 = zeros(nr,na);

rrm = r/rm;
e1 = exp((rm - r)/L); 

ix = find(r ~= -999.0);  % Normal points

v1 = vm*rrm;
dv1(:,1) = rrm;
dv1(:,2) = 0;
dv1(:,3) = -vm*rrm/rm;

v2 = vm*e1;
dv2(:,1) = e1;
dv2(:,2) = -vm*(rm - r).*e1/L^2;
dv2(:,3) = vm*e1/L;

v = (1 - w).*v1 + w.*v2;
dv = (1 - w*oa).*dv1 + w*oa.*dv2 + dw.*((v2 - v1)*oa);

ix = find(r == -999.0);
if ~isempty(ix) 
   % Special point, put in penalty term on length scales etc.
   rmhat = 25e3; rmscal = 4;
   Lhat = 5e4;  Lscal = 1;
   L2hat = 3e5;  L2scal = 1;
   v(ix) = rmscal*(rm/rmhat - 1)^2 ...
      + Lscal*(L/Lhat - 1)^4 ...
      + L2scal*(L2/L2hat - 1)^4;
   dv(ix,1) = 0;
   dv(ix,2) = 4*Lscal/Lhat*(L/Lhat - 1)^3;
   dv(ix,3) =   rmscal/rmhat*(rm/rmhat - 1);
end


