function [p,dp] = willoughby2_p_num_fixblend(r,a,f);
% function [p,dp] = willoughby2_p_num(r,a,f);
% Returns the double-exponential parametric wind profile of Willoughby (2002)
% at radius points r for control vector a=(A,L1,B,L2,rm) and its derivatives
% with respect to the control vector together with the radial integral of the
% gradient wind equation i.e. the corresponding pressure field.
% The centre of the blending zone is set through the global variable w2_p_rblend 
% rather than calculated from the input parameters.
% Uses willoughby2_v to get the wind field and then numerically integrates it.
% The control vector a = (A,L1,B,L2,rm,n1,pc,T,Lb)
% Note derivatives wrt T and Lb not implemented.

global w2_fix_rblend

if (nargin < 3)
   global f0;
   f = f0;
   if isempty(f)
      disp('Must define Coriolis parameter');
      p = nan; dp = nan;
      return
   end
end

if isempty(w2_fix_rblend)
   disp('Must define global w2_fix_rblend')
   p = nan; dp = nan;
   return
end

A  = a(1);   % amplitude of first exponential
L1 = a(2);   % length scale of first exponential
B  = a(3);   % amplitude of second exponential
L2 = a(4);   % length scale of second exponential
rm = a(5);   % radius of maximum winds
n1 = a(6);   % exponent for centre profile (v1)
pc = a(7);   % central pressure
T  = a(8);   % for density terms
if length(a) > 8
   Lb = a(9);   % Half-width of blending zone
else
   Lb = 5e3;
end
R  = 287;
RT = R*T;
RTi = 1/RT;
rblend1 = w2_fix_rblend - Lb;
rblend2 = w2_fix_rblend + Lb;

if (A < -10 | L1 <= 0 | B < -10 | L2 <= 0 | rm <= 0 | n1 <= 0 | rblend1 <= 0)
   disp('*** willoughby2_p_num_fixblend >> bad a '); disp(a(:)); disp(w2_fix_rblend);
   p = nan; dp = nan;
   % pause
   return
end

L1i = 1/L1; L2i = 1/L2;
L12i = L1i + L2i;

rs = ( r - w2_fix_rblend )/Lb;   % Scaled r for weight
rrm = r/rm;

rmax = max(r);
nr = length(r);
p  = zeros(nr,1) + nan;
if nargout >= 2
   dp = zeros(nr,10) + nan;
end

% Centre
ix = find( r == 0 );
if ~isempty(ix)
   p(ix) = pc;
   if 0 & nargout >= 2
      dp(ix, 1) = 0;
      dp(ix, 2) = 0;
      dp(ix, 3) = 0;
      dp(ix, 4) = 0;
      dp(ix, 5) = 0;
      dp(ix, 6) = 0;
      dp(ix, 7) = 1;
      dp(ix, 8) = 0;
      dp(ix, 9) = 0;
      dp(ix,10) = 0;
   end
end

% Inner zone - from r=0 to start of blending
ix = find( 0 < r & r <= rblend1 );
if ~isempty(ix)
   rrmn = rrm(ix).^n1;
   gwe1 = ((A + B)/(2*n1*(n1 + 1))) ...
      * (2*f*n1*r(ix) + (A + B)*(1 + n1)*rrmn).*rrmn;
   ee1 = exp(RTi*gwe1);
   p(ix) = pc*ee1;
   if nargout >= 2
      dp(ix, 1) = (RTi/(n1*(n1 + 1))) * (f*n1*r(ix) + (A + B)*(n1 + 1)*rrmn).*rrmn.*p(ix);
      dp(ix, 2) = 0;
      dp(ix, 3) = dp(ix,1);
      dp(ix, 4) = 0;
      dp(ix, 5) = (-RTi*(A + B)/((n1 + 1)*rm)) * (f*n1*r(ix) + (A + B)*(n1 + 1)*rrmn).*rrmn.*p(ix);
      dp(ix, 6) = ((A + B)*RTi*rrmn/(2*(n1 + n1^2)^2)) .* ...
         (-2*f*n1^2*r(ix) - (A + B)*(n1 + 1)^2*rrmn ...
           + 2*n1*(n1 + 1)*(f*n1*r(ix) + (A + B)*(n1 + 1)*rrmn).*log(rrm(ix))).*p(ix);
      dp(ix, 7) = ee1;
      dp(ix, 8) = -gwe1 .* p(ix) / (R*T^2);
      dp(ix, 9) = 0;
      dp(ix,10) = (A + B)*RTi*(f + (A + B)*rrmn./r(ix)).*rrmn.*p(ix);
   end
end

rrmn = (rblend1/rm).^n1;
gwe1 = ((A + B)/(2*n1*(n1 + 1))) * (2*f*n1*rblend1 + (A + B)*(1 + n1)*rrmn).*rrmn;
ee1 = exp(RTi*gwe1);
pblend1 = pc*ee1;
% pblend1 = pc*exp(RTi*(((A + B)^2/(2*n1)*(rblend1/rm)^(2*n1) + f*(A + B)*rm/(n1+1)*(rblend1/rm)^(n1+1))));

dpblend1 = zeros(1,10);
dpblend1( 1) = (RTi/(n1*(n1 + 1))) * (f*n1*rblend1 + (A + B)*(n1 + 1)*rrmn)*rrmn*pblend1;
dpblend1( 2) = 0;
dpblend1( 3) = dpblend1(1);
dpblend1( 4) = 0;
dpblend1( 5) = (-RTi*(A + B)/((n1 + 1)*rm)) * (f*n1*rblend1 + (A + B)*(n1 + 1)*rrmn)*rrmn*pblend1;
dpblend1( 6) = ((A + B)*RTi*rrmn/(2*(n1 + n1^2)^2)) * ...
         (-2*f*n1^2*rblend1 - (A + B)*(n1 + 1)^2*rrmn ...
           + 2*n1*(n1 + 1)*(f*n1*rblend1 + (A + B)*(n1 + 1)*rrmn)*log(rblend1/rm))*pblend1;
dpblend1( 7) = ee1;
dpblend1( 8) = -gwe1 * pblend1 / (R*T^2);
dpblend1( 9) = 0;
dpblend1(10) = (A + B)*RTi*(f + (A + B)*rrmn/rblend1)*rrmn*pblend1;

% Blending zone
imode = 'linear';
ix = find( rblend1 < r & r <= rblend2 );
ndum =  500;
rdum = ([ 0 : ndum ]')*2*Lb/ndum + rblend1; drdum = rdum(2) - rdum(1);
[vdum, dvdum] = willoughby2_v_fixblend(rdum, a);
gwe = RTi * vdum .* ( vdum./rdum + f );
intgwe = cumint(gwe, drdum); 
edum = exp(intgwe);
pdum = pblend1*edum;
pblend2 = pdum(ndum+1);
dpblend2 = zeros(1,10);
if ~isempty(ix)
   p(ix) = interp1(rdum,pdum,r(ix),imode);
end
if nargout >= 2      % Calculate derivatives
   Iterm = RTi*(2*vdum./rdum + f);
   for ii = 1 : 7
      dpdum = pdum .* cumint( Iterm .* dvdum(:,ii), drdum ) ...
            + edum * dpblend1(ii);
      dpblend2(ii) = dpdum(ndum+1);
      if ~isempty(ix)
         dp(ix,ii) = interp1(rdum,dpdum,r(ix),imode);
      end
   end
   dpdum = -pdum .* intgwe / T + edum*dpblend1(8);
   dpblend2(8) = dpdum(ndum+1);
   dpblend2(9) = dpdum(ndum+1);
   dpblend2(10) = pdum(ndum+1)*gwe(ndum+1);
   if ~isempty(ix)
      dp(ix,8) = nan; % interp1(rdum,dpdum,r(ix),imode);
      dp(ix,9) = 0;
      dp(ix,10) = interp1(rdum,pdum.*gwe, r(ix),imode);
   end
end

% Outer zone
ix = find( rblend2 < r );
if ~isempty(ix)
   Ei1  = -real(expint(2*r(ix)*L1i)) + real(expint(2*rblend2*L1i));
   Ei2  = -real(expint(2*r(ix)*L2i)) + real(expint(2*rblend2*L2i));
   Ei12 = -real(expint(r(ix)*L12i)) + real(expint(rblend2*L12i));
   e1m  = min(exp(rm*L1i), 1e100); % Avoid overflow
   e2m  = min(exp(rm*L2i), 1e100); 
   e12m = min(exp(rm*L12i),1e100); 
   e1 = exp((rm - r(ix))*L1i); e1b = exp((rm - rblend2)*L1i);
   e2 = exp((rm - r(ix))*L2i); e2b = exp((rm - rblend2)*L2i);
   e12 = exp((rm - r(ix))*L12i); e12b = exp((rm - rblend2)*L12i);
   Ei1e1m2 = (Ei1*e1m)*e1m;  % Avoid overflow when L1 small
   [vdum, dvdum] = willoughby2_v_fixblend(r(ix), a);
   gwe = RTi*( ( vdum./r(ix) + f ) .* vdum );
   intgwe = RTi*( f*( A*L1*(e1b - e1) + B*L2*(e2b - e2) ) ...
      + A^2 * Ei1e1m2 + (B*e2m)^2 * Ei2 + 2*A*B * e12m * Ei12 );
  % [ intgwe, Ei1,e1m,Ei1e1m2 , (B*e2m)^2 * Ei2 , 2*A*B * e12m * Ei12]
   egwe = exp(intgwe);
   p(ix) = pblend2 * egwe;
   if nargout >= 2
      dp(ix,1) = dpblend2(1)*egwe ...
               + p(ix) .* ( RTi*( f*L1*(e1b - e1) + 2*A*Ei1e1m2 + 2*B*e12m*Ei12 ) );
      dp(ix,3) = dpblend2(3)*egwe ...
               + p(ix) .* ( RTi*( f*L2*(e2b - e2) + 2*B*e2m^2*Ei2 + 2*A*e12m*Ei12 ) );
      dp(ix,2) = dpblend2(2)*egwe ...
               + p(ix) *  RTi  ...
                      .* ( (A/L1)^2 * ( (e1b^2 - e1.^2)*L1 - 2*rm*Ei1e1m2 ) ...
                         + A*f/L1 * (e1b*(L1 + rblend2 - rm) - e1.*(L1 + r(ix) - rm)) ...
                         - 2*A*B/(L1^2*(L1+L2)) * (L1*L2*(e12 - e12b) +e12m*(L1+L2)*rm*Ei12) );
      dp(ix,4) = dpblend2(4)*egwe ...
               + p(ix) *  RTi  ...
                      .* ( (B/L2)^2 * ( (e2b^2 - e2.^2)*L2 - 2*e2m^2*rm*Ei2 ) ...
                         + B*f/L2 * (e2b*(L2 + rblend2 - rm) - e2.*(L2 + r(ix) - rm)) ...
                         - 2*A*B/(L2^2*(L1+L2)) * (L1*L2*(e12 - e12b) +e12m*(L1+L2)*rm*Ei12) );
      dp(ix,5) = dpblend2(5)*egwe ...
               + p(ix) * RTi  ...
                      .* ( 2*A^2*Ei1e1m2/L1 + 2*(B*e2m)^2*Ei2/L2 ...
                         + f*(B*(e2b-e2) + A*(e1b-e1)) + 2*A*B*e12m*Ei12*L12i );
      dp(ix,6) = dpblend2(6)*egwe;
      dp(ix,7) = dpblend2(7)*egwe;
      dp(ix,8) = dpblend2(8)*egwe - p(ix).*intgwe/T;
      dp(ix,9) = 0; 
   end
   dp(ix,10) = p(ix) .* gwe;
end

% Special points
ix = find(r == -999.0);
if ~isempty(ix)
   global w2p_rmhat w2p_rmscal w2p_L1hat w2p_L1scal w2p_L2hat w2p_L2scal
   %  w2p_rmhat = 25e3; w2p_rmscal = 1;
   %  w2p_L1hat = 7e4;  w2p_L1scal = 1;
   %  w2p_L2hat = 4e5;  w2p_L2scal = 1;
   if (  ~isempty(w2p_rmhat) & ~isempty(w2p_rmscal) ...
       & ~isempty(w2p_L1hat) & ~isempty(w2p_L1scal) ...
       & ~isempty(w2p_L2hat) & ~isempty(w2p_L2scal) )
      p(ix) = w2p_rmscal*(rm/w2p_rmhat - 1)^4 ...
            + w2p_L1scal*(L1/w2p_L1hat - 1)^4 ...
            + w2p_L2scal*(L2/w2p_L2hat - 1)^4;
      if nargout >= 2
         dp(ix, 1) = 0;
         dp(ix, 2) = 4*w2p_L1scal/w2p_L1hat*(L1/w2p_L1hat - 1)^3;
         dp(ix, 3) = 0;
         dp(ix, 4) = 4*w2p_L2scal/w2p_L2hat*(L2/w2p_L2hat - 1)^3;
         dp(ix, 5) = 4*w2p_rmscal/w2p_rmhat*(rm/w2p_rmhat - 1)^3;
         dp(ix, 6) = 0;
         dp(ix, 7) = 0;
         dp(ix, 8) = 0;
         dp(ix, 9) = 0;
         dp(ix,10) = 0;
      end
   else
      disp('*** Global variables w2p_* undefined in willoughby2_p_num_fixblend');
      p(ix) = nan;
   end
end




