function [p] = willoughby2_p_numerical(r,a,f);
% function [p] = willoughby2_p_numerical(r,a,f);
% function [p,dp] = willoughby2_p_numerical(r,a,f);
% Returns the double-exponential parametric wind profile of Willoughby (2002)
% at radius points r for control vector a=(A,L1,B,L2,rm) and its derivatives
% with respect to the control vector together with the radial integral of the
% gradient wind equation i.e. the corresponding pressure field.
% Uses willoughby2_v to get the wind field and then numerically integrates it.
% The control vector a = (A,L1,B,L2,rm,n1,pc,T)

persistent olda A L1 L1i B L2 L2i rm n1 pc T R RT RTi Lb L12i
persistent wm dwmdA dwmdB dwmdL1 dwmdL2 dwmdrm 
persistent rsm dwsdrm_rsm dwsdrsi_rsm rblend1 rblend2

if (nargin < 3)
   global f0;
   f = f0;
   if isempty(f)
      disp('Must define Coriolis parameter');
      return
   end
end

if isempty(olda) | any(size(a) ~= size(olda)) | any(a ~= olda)  
   % Initialise on first time or changed a
   olda = a;
   
   A = a(1);    % amplitude of first exponential
   L1 = a(2);   % length scale of first exponential
   B = a(3);    % amplitude of second exponential
   L2 = a(4);   % length scale of second exponential
   rm = a(5);   % radius of maximum winds
   n1 = a(6);   % exponent for centre profile (v1)
   pc = a(7);   % central pressure
   T = a(8);  % for density terms
   if length(a) > 8
      Lb = a(9);   % Half-width of blending zone
   else
      Lb = 5e3;
   end
   R = 287;
   RT = R*T;
   RTi = 1/RT;
   L1i = 1/L1; L2i = 1/L2;
   L12i = L1i + L2i;
   
   % Find wm, the weight that must apply at rm, and its derivatives.
   wm = 1 / ( 1 + rm*(A/L1 + B/L2)/(A + B) );
   dwmdA  =  wm^2 * B*rm*(1/L2 - 1/L1)/(A + B)^2;
   dwmdB  =  wm^2 * A*rm*(1/L1 - 1/L2)/(A + B)^2;
   dwmdL1 =  wm^2 * A*rm/((A + B)*L1^2);
   dwmdL2 =  wm^2 * B*rm/((A + B)*L2^2);
   dwmdrm = -wm^2 * (A/L1 + B/L2)/(A + B); 
   
   % Find the scaled radius rsm = rs(rm) satisfying ws(rsm) = wm, by bisection
   rsm = willoughby2_rsm(a);
   
 % dwsdrs_rsm = 0.9375*(rsm^2 - 1)^2; commented out 19.1.2005 seemingly unused
 % dwsdrsi_rsm = 1/dwsdrs_rsm;
   
   rblend1 = rm - Lb*rsm - Lb;
   rblend2 = rm - Lb*rsm + Lb;
end

if rblend1 < 0
   disp(sprintf('WARNING willoughby2_p_numerical >> rblend1 = %f < 0', rblend1));
   p = repmat(nan, size(r));
   return;
end

% rho = pc/RT;
rs = ( r - rm )/Lb + rsm;   % Scaled r
rrm = r/rm;

rmax = max(r);
nr = length(r);
p  = zeros(nr,1) + nan;
if nargout >= 2
   dp = zeros(nr,9) + nan;
end

% Centre
ix = find( r == 0 );
if ~isempty(ix)
   p(ix) = pc;
   if 0 & nargout >= 2
      dp(ix,1) = 0;
      dp(ix,2) = 0;
      dp(ix,3) = 0;
      dp(ix,4) = 0;
      dp(ix,5) = 0;
      dp(ix,6) = 0;
      dp(ix,7) = 1;
      dp(ix,8) = 0;
      dp(ix,9) = 0;
   end
end

% Inner zone - from r=0 to start of blending
ix = find( 0 < r & r <= rblend1 );
if ~isempty(ix)
   rrmn = rrm(ix).^n1;
   gwe1 = ((A + B)/(2*n1*(n1 + 1))) ...
      * (2*f*n1*r(ix) + (A + B)*(1 + n1)*rrmn).*rrmn;
   ee1 = exp(RTi*gwe1);
   p(ix) = pc*ee1;
   if 0 & nargout >= 2
      dp(ix,1) = (RTi/(n1*(n1 + 1))) * (f*n1*r(ix) + (A + B)*(n1 + 1)*rrmn).*rrmn.*p(ix);
      dp(ix,2) = 0;
      dp(ix,3) = dp(ix,1);
      dp(ix,4) = 0;
      dp(ix,5) = (-RTi*(A + B)/((n1 + 1)*rm)) * (f*n1*r(ix) + (A + B)*(n1 + 1)*rrmn).*rrmn.*p(ix);
      dp(ix,6) = ((A + B)*RTi*rrmn/(2*(n1 + n1^2)^2)) .* ...
         (-2*f*n1^2*r(ix) - (A + B)*(n1 + 1)^2*rrmn ...
           + 2*n1*(n1 + 1)*(f*n1*r(ix) + (A + B)*(n1 + 1)*rrmn).*log(rrm(ix))).*p(ix);
      dp(ix,7) = ee1;
      dp(ix,8) = -gwe1 .* p(ix) / (R*T^2);
      dp(ix,9) = (A + B)*RTi*(f + (A + B)*rrmn./r(ix)).*rrmn.*p(ix);
   end
end

pblend1 = pc*exp(RTi*(((A + B)^2/(2*n1)*(rblend1/rm)^(2*n1) + f*(A + B)*rm/(n1+1)*(rblend1/rm)^(n1+1))));

% Blending zone
ix = find( rblend1 < r & r <= rblend2 );
ndum = 50;
rdum = ([ 0 : ndum ]')*2*Lb/ndum + rblend1; drdum = rdum(2) - rdum(1);
[vdum, dvdum] = willoughby2_v(rdum, a(1:9));
gwe = vdum .* ( vdum./rdum + f );
pdum = pblend1*exp(RTi*(cumint(gwe, drdum)));
if ~isempty(ix)
   p(ix) = interp1(rdum,pdum,r(ix),'spline');
   if 0 & (nargout >= 2)  % Calculate derivatives
      Iterm = 2*vdum./rdum + f;
      for ii = 1 : 8
         dpdum = pdum .* ( RTi*cumint( Iterm .* dvdum(:,ii), drdum ) );
         dp(ix,ii) = interp1(rdum,dpdum,r(ix),'spline');
      end
    % dp(ix,6) = 1;
   end
end

pblend2 = pdum(ndum+1);

% Outer zone
ix = find( rblend2 < r );
Ei1  = -real(expint(2*r(ix)*L1i)) + real(expint(2*rblend2*L1i));
Ei2  = -real(expint(2*r(ix)*L2i)) + real(expint(2*rblend2*L2i));
Ei12 = -real(expint(r(ix)*L12i)) + real(expint(rblend2*L12i));
e1m = exp(rm*L1i); e1 = exp((rm - r(ix))*L1i); e1b = exp((rm - rblend2)*L1i);
e2m = exp(rm*L2i); e2 = exp((rm - r(ix))*L2i); e2b = exp((rm - rblend2)*L2i);
e12m = exp(rm*L12i); e12 = exp((rm - r(ix))*L12i); e12b = exp((rm - rblend2)*L12i);
%p(ix) = pblend2 + rho*( f*( A*L1*(e1b - e1) + B*L2*(e2b - e2) ) ...
%   + (A*e1m)^2 * Ei1 + (B*e2m)^2 * Ei2 + 2*A*B * e12m * Ei12 );
p(ix) = pblend2*exp(RTi*( f*( A*L1*(e1b - e1) + B*L2*(e2b - e2) ) ...
   + (A*e1m)^2 * Ei1 + (B*e2m)^2 * Ei2 + 2*A*B * e12m * Ei12 ));
if 0
   dp(ix,1) = rho*( (A + B + 0.5*f*rm) + f*L1*(1 - e1) + 2*A*e1m^2*Ei1 + 2*B*e12m*Ei12 );
   dp(ix,3) = rho*( (A + B + 0.5*f*rm) + f*L2*(1 - e2) + 2*B*e2m^2*Ei2 + 2*A*e12m*Ei12 );
   dp(ix,2) = rho*A*( f*( 1 - e1.*( 1 - L1i*(rm - r(ix)) ) ) ...
      + A*L1i*( (1 - e1.^2) - 2*rm*L1i*e1m^2*Ei1 ) ...
      + 2*B*L1i*( L2*(1 - e12)/(L1+L2) - rm*e12m*L1i*Ei12 ) );
   dp(ix,4) = rho*B*( f*( 1 - e2.*( 1 - L2i*(rm - r(ix)) ) ) ...
      + B*L2i*( (1 - e2.^2) - 2*rm*L2i*e2m^2*Ei2 ) ...
      + 2*A*L2i*( L1*(1 - e12)/(L1+L2) - rm*e12m*L2i*Ei12 ) );
   dp(ix,5) = rho*( 0.5*f*(A + B) - f*(A*e1 + B*e2) - ((A + B)^2)/rm ...
      + 2*(A*e1m)^2*L1i*Ei1 ...
      + 2*(B*e2m)^2*L2i*Ei2 ...
      + 2*A*B*e12m*L12i*Ei12 );
   dp(ix,6) = 1;
end

% Special points
ix = find(r == -999.0);
if ~isempty(ix)
   global w2p_rmhat w2p_rmscal w2p_L1hat w2p_L1scal w2p_L2hat w2p_L2scal
   w2p_rmhat = 25e3; w2p_rmscal = 1;
   w2p_L1hat = 7e4;  w2p_L1scal = 1;
   w2p_L2hat = 4e5;  w2p_L2scal = 1;
   p(ix) = w2p_rmscal*(rm/w2p_rmhat - 1)^4 ...
      + w2p_L1scal*(L1/w2p_L1hat - 1)^4 ...
      + w2p_L2scal*(L2/w2p_L2hat - 1)^4;
   if 0
      dp(ix,1) = 0;
      dp(ix,2) = 4*w2p_L1scal/w2p_L1hat*(L1/w2p_L1hat - 1)^3;
      dp(ix,3) = 0;
      dp(ix,4) = 4*w2p_L2scal/w2p_L2hat*(L2/w2p_L2hat - 1)^3;
      dp(ix,5) = 4*w2p_rmscal/w2p_rmhat*(rm/w2p_rmhat - 1)^3;
      dp(ix,6) = 0;
      dp(ix,7) = 0;
      dp(ix,8) = 0;
      dp(ix,9) = nan;
   end
end

%   opt = odeset('AbsTol',0.1, 'OutputFcn',@odeplot, 'Stats','on', ...
%                 'InitialStep',1e3);
%   [rr,pp] = ode45(@gwe, rr, pc, opt, a, f);
%   p(ix) = pp(iperm);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%function dpdr = gwe(r,p,a,f)

%rho = 1.15;
%v =  willoughby2_v(r, a(1:5));
%dpdr = rho .* v .* ( v./(r + 1e-100) + f );


