function [v,dv,wzone] = willoughby2_v(r,a)
% function [v,dv,wzone] = willoughby2_v(r,a)
% Returns the double-exponential parametric wind profile of Willoughby (2002)
% at radius points r for control vector a=(A,L1,B,L2,rm,n1,pc,T, Lb) and its derivatives
% with respect to the control vector. Also returns wzone which contains the
% radii at which the blending starts and finishes.
% Nov 2005 added derivative wrt Lb and r in dv (previously dp had 8 cols, now 10)

persistent olda A L1 B L2 rm n1 Lb wm dwmdA dwmdB dwmdL1 dwmdL2 dwmdrm dwmdn1 
persistent rsm dwsdrm_rsm dwsdrsi_rsm

if isempty(olda) | any(size(a) ~= size(olda)) | any(a ~= olda)  
   % Initialise on first time or changed a
   olda = a;
   
   A = a(1);    % amplitude of first exponential
   L1 = a(2);   % length scale of first exponential
   B = a(3);    % amplitude of second exponential
   L2 = a(4);   % length scale of second exponential
   rm = a(5);   % radius of maximum winds
   n1 = a(6);   % Exponent for v1 (inner zone)
   pc = a(7);   % central pressure (not used)
   RT = 287*a(8);  % for density terms (not used)
%  if length(a) > 8   % default for short a removed 16 Nov 2005
      Lb = a(9);   % Half-width of blending zone
%  else
%     Lb = 5e3;
%  end
   
   % Find wm, the weight that must apply at rm, and its derivatives.
   wm = (A + B)*L1*L2*n1 / (A*L2*(L1*n1 + rm) + B*L1*(L2*n1 + rm));
   dwmdA  =  wm^2 * B*rm*(1/L2 - 1/L1)/(n1*(A + B)^2);
   dwmdB  =  wm^2 * A*rm*(1/L1 - 1/L2)/(n1*(A + B)^2);
   dwmdL1 =  wm^2 * A*rm/(n1*(A + B)*L1^2);
   dwmdL2 =  wm^2 * B*rm/(n1*(A + B)*L2^2);
   dwmdrm = -wm^2 * (A/L1 + B/L2)/(n1*(A + B)); 
   dwmdn1 =  wm^2 * ((A/L1 + B/L2)*rm)/(n1^2*(A + B));
   
   % Find the scaled radius rsm = rs(rm) satisfying ws(rsm) = wm, by bisection
   rsm = willoughby2_rsm(a);

   if ( rm - Lb*rsm - Lb  < 0 )
      disp(sprintf('WARNING willoughby2_v >> rblend1 = %f < 0, rsm adjusted', rm - Lb*rsm - Lb));
      rsm = rm/Lb - 1;
   end
   
 % dwsdrs_rsm = 0.9375*(rsm^2 - 1)^2;
   dwsdrs_rsm = (315/256)*(rsm^2 - 1)^4;
   dwsdrsi_rsm = 1/dwsdrs_rsm;
end

rs = ( r - rm )/Lb + rsm;   % Scaled r for blending

nr  = length(r); na = 9;

w  = zeros(size(r)); 

ix = find(abs(rs) < 1);
%% Old version has 2 continuous derivatives, new one has 4. See also dwsdrs_rs
%% w(ix) = 0.5 + 0.0625*rs(ix).*(3*rs(ix).^4 - 10*rs(ix).^2 + 15);
w(ix) = (1/256) * (1 + rs(ix)).^5 .* (128 - rs(ix).*(325 - rs(ix).*(345 - rs(ix).*(175 - rs(ix)*35))));
w( find(rs >= 1) ) = 1;

% fig(4); clf; plot(r,w);

rrm = r/rm;
e1 = exp((rm - r)/L1);
e2 = exp((rm - r)/L2);

ix = find(r ~= -999.0);  % Normal points

rrmn = rrm.^n1;
v1 = (A + B)*rrmn;
v2 = A*e1 + B*e2;
v = (1 - w).*v1 + w.*v2;

if nargout >= 2
   ix = find(abs(rs) < 1);
 % dwsdrs_rs = 0.9375*(rs(ix).^2 - 1).^2;
   dwsdrs_rs = (315/256)*(rs(ix).^2 - 1).^4;
   term = dwsdrs_rs * dwsdrsi_rsm;
   dw  = zeros(nr,na+1);
   dw(ix, 1) = term * dwmdA; 
   dw(ix, 2) = term * dwmdL1;
   dw(ix, 3) = term * dwmdB;
   dw(ix, 4) = term * dwmdL2;
   dw(ix, 5) = term * dwmdrm - dwsdrs_rs/Lb;
   dw(ix, 6) = term * dwmdn1;
   dw(ix, 7) = 0;
   dw(ix, 8) = 0;
   dw(ix, 9) = nan;
   dw(ix,10) = dwsdrs_rs/Lb;
   dw( find(rs >= 1) ,:) = 0;

   dv1 = zeros(nr,na+1);
   dv1(:, 1) = rrmn;
   dv1(:, 2) = 0;
   dv1(:, 3) = rrmn;
   dv1(:, 4) = 0;
   dv1(:, 5) = -(n1/rm)*v1;
   ix = find(r >  0); dv1(ix, 6) = log(rrm(ix)).*v1(ix);
   ix = find(r == 0); dv1(ix, 6) = 0;
   dv1(:, 7) = 0;
   dv1(:, 8) = 0;
   dv1(:, 9) = 0;
   dv1(:,10) = ((A + B)*n1/rm) * rrm.^(n1-1);
   
   dv2 = zeros(nr,na+1);
   dv2(:, 1) = e1;
   dv2(:, 2) = -A*(rm - r).*e1/L1^2;
   dv2(:, 3) = e2;
   dv2(:, 4) = -B*(rm - r).*e2/L2^2;
   dv2(:, 5) = A*e1/L1 + B*e2/L2;
   dv2(:, 6) = 0;
   dv2(:, 7) = 0;
   dv2(:, 8) = 0;
   dv2(:, 9) = 0;
   dv2(:,10) = -(A/L1)*e1 - (B/L2)*e2;

   oa = ones(1,na+1);
   dv = (1 - w*oa).*dv1 + w*oa.*dv2 + dw.*((v2 - v1)*oa);
end

ix = find(r == -999.0);
if ~isempty(ix) 
   % Special point, put in penalty term on length scales etc.
   global w2v_rmhat w2v_rmscal w2v_L1hat w2v_L1scal w2v_L2hat w2v_L2scal 
   v(ix) = w2v_rmscal*(rm/w2v_rmhat - 1)^2 ...
      + w2v_L1scal*(L1/w2v_L1hat - 1)^4 ...
      + w2v_L2scal*(L2/w2v_L2hat - 1)^4;
   if nargout >= 2
      dv(ix,1) = 0;
      dv(ix,2) = 4*w2v_L1scal/w2v_L1hat*(L1/w2v_L1hat - 1)^3;
      dv(ix,3) = 0;
      dv(ix,4) = 4*w2v_L2scal/w2v_L2hat*(L2/w2v_L2hat - 1)^3;
      dv(ix,5) =   w2v_rmscal/w2v_rmhat*(rm/w2v_rmhat - 1);
      dv(ix,6) = 0;
      dv(ix,7) = 0;
      dv(ix,8) = 0;
   end
end


