function [p,v] = willoughby_pv(beta, r, f)
% Based on Hugh's Fortran code.

if (nargin <= 2)
   global f0
   f = f0; % cor(15);
end

ENI = 2.0;
Vmax = beta(1);
Rmax = beta(2);
WID = 10e3;
Xlen = beta(3);
pc = beta(4);

[R1,R2] = FindR1R2(Rmax,WID,ENI,Xlen);
v = VofR(r,Vmax,Rmax,R1,R2,WID,ENI,Xlen);

RT = 287*298;
drdum = 1e3;
rdum = 0.1 + [ 0 : 500 ] * drdum;
vdum = VofR(rdum,Vmax,Rmax,R1,R2,WID,ENI,Xlen);
gwe = vdum.^2 ./ rdum + f*vdum;
intgwe = drdum * (cumsum(gwe) - 0.5*gwe - 0.5*gwe(1));
pdum = pc * exp( intgwe / RT );
p = interp1( rdum,pdum,r );


   %---------------------------------------------------
	%Library name Simple_lib.for
	%Written by H. E. Willoughby 8 May 2001
	%Latest update 9 May 2001 by HEW
	%Function: Subroutines to compute hurricane tangential wind 
	%           as a function of radius. Uses single exponential version 
	%           of the profile.

	
function [R1,R2] = FindR1R2(Rmax,WID,ENI,Xlen)
	%-----------------------------------------------
	%Finds inner and outer boundary of transition region

	% Compute weight at radius of maximum wind
	w = 1.0/(1.0 + Rmax/(ENI*Xlen));

	%Compute nondimensional coordinate of that weight 
	%by interval bisection.
	x = findx(w);

	%Approximate nondimensional coordinate uning asyptotic cubic parabola
	% x2 = 1.0 - (0.1*(1-w))**0.333333

	%Write results to putput file
	%write(14,'(1x,3f8.3)')w,x,x2

	%Compute inner boundary 
	R1 = Rmax - x*WID;

	%Compute outer boundary
	R2 = R1 + WID;

function w = wgt(x)
	%----------------------------------------------------
	%Polynomial ramp function: when x, the nondimensional 
	%argument, is zero wgt=0; when x = 1 wgt =1, First and 
	%second derivitives arezero at both ends, C2 continuity.
   w = zeros(size(x)) + nan;
   ix = find(x <= 0); w(ix) = 0;
   ix = find(0 < x & x <= 1); 
   w(ix) = x(ix).^3 .* (10.0 - x(ix).*(15.0 - 6.0*x(ix)));
   ix = find(x >= 1); w(ix) = 1;

function vv = VofR(RR,Vmax,Rmax,R1,R2,WID,ENI,Xlen)
	%Sectionally continuous wind-profile function with a 
	%power law inside the eye and a single-length exponential
	%decay outside. There is a smooth polynimial transition
	%across the radius of maximum wind.
	 
	w = wgt((RR - R1)/WID); %  wheighting function
   %clf; plot(RR,dx2(w)); xlim(1e4,2.5e4);
	%Compute weighted combination of inside and outside profiles
	vv = (1.0-w).*Vin(RR,Vmax,Rmax,ENI) + w.*Vout(RR,Vmax,Rmax,Xlen);

function vv = Vin(RR,Vmax,Rmax,ENI)
	%Power-law wind profile for inside the eye
	vv = Vmax*(RR/Rmax).^ENI;

function vv = Vout(RR,Vmax,Rmax,Xlen)
	%Single-length exponential for outside the eye
	vv = Vmax*exp(-(RR - Rmax)/Xlen);

function XX = findx(WW)
%---------------------------------------------------------------
% Uses interval bisection to find coordinate XX of a given 
% weighting WW on the polynomial ramp function

   eps0 = 0.0001;

	%Initialize interval
   done = 0;
   X1=0.0;		%Lower
   W1 = wgt(X1);
   X2=1.0;		%Upper
   W2 = wgt(X2);

	%Iterate
   while (done == 0)
      
      %Find value at midpoint
      X3=0.5*(X1+X2);
      W3 = wgt(X3);
      
      %Which half contains zero crossing?
      if (W3 < WW)
         X1 = X3;
         W1 = W3;
      else
         X2 = X3;
         W2 = W3;
      end
      XX=X3;
      
      %Converged yet?
      done = (abs(X1-X2) < eps0);
   end
   %---------------------------------------
	%VARIABLE DICTIONARY
	%dR		Radial mesh interval (km)
	%ENI	Power-law exponent for porfile inside the eye
	%r		Array for radius (km)
	%R1		Inner boundary of transition region (km)
	%R2		Outer boundary of transition region (km)
	%Rmax	Radius of maximum wind (km)
	%v		Array for tangential wind (km)
	%Vmax	Maximum wind (m/s) 
	%WID	Width of transition region (km)
	%Xlen	Exponential decay length in outer vortex (km)
